@@ -1,10 +1,12 @@
 import json
 
 import requests
+import datetime
 
 from .exceptions.exceptions import IncidentNotExistsException, ProcessingStatusDoesNotExistException
 
 
+
 class TopdeskConnector:
 	def __init__(self, baseURL: str, authkey: str):
 		"""
@@ -18,7 +20,8 @@ class TopdeskConnector:
 		}
 		self.baseURL = baseURL
 
-	def make_request(self, method, url_extension, params=None, headers=None, files=None, json_data=None, auto_parse=True):
+	def make_request(self, method, url_extension, params=None, headers=None, files=None, json_data=None,
+					 auto_parse=True):
 		"""
 		Make a request to the API and return the repsonse content
 
@@ -99,6 +102,15 @@ class TopdeskConnector:
 	# 	event_incidents = self.__incidents(params=params)
 	# 	return event_incidents
 
+	def get_incident_call_types(self):
+		"""
+		Returns a list of incident call types in the following format
+		[
+			{"id": "8e1ae455-88ba-493d-a110-99e0b0f8ee69", "name": "Complaint" },
+		]
+		"""
+		return self.make_request('get', 'incidents/call_types')
+
 	def incident_status(self, incident_id: str):
 		"""
 		Return de status string van een incident
@@ -193,18 +205,21 @@ class TopdeskConnector:
 		incident = self.incident(incident_id)
 		return incident.get('callType').get('name')
 
-	def incidents_supplier(self, supplier_id, amount=5):
+	def incidents_by_supplier(self, supplier_id, completed: bool or None = None):
 		"""
 		Get all incidents linked to a supplier id
-		:param amount: amount of incidents to request.
 		:param supplier_id: string
+		:param completed: bool or None
 		:return:
 		"""
 		# naam: Comparex Nederland B.V.
-		params = {
-			'supplier': { "id": f"{supplier_id}" },
-			'page_size': amount
-		}
+		if completed is None:
+			params = f'query=supplier.id=={supplier_id}'
+		if completed:
+			params = f'query=supplier.id=={supplier_id};completed==True'
+		if completed is False:
+			params = f'query=supplier.id=={supplier_id};completed==False'
+
 		return self.make_request('GET', "incidents", params=params)
 
 	def newest_incidents(self, amount=5, start_offset=0, processing_status_id: str = None):
@@ -323,8 +338,10 @@ class TopdeskConnector:
 				filename = attachment.get('fileName')
 				break
 		if filename is None:
-			raise FileNotFoundError("Attachment is niet gevonden tussen de lijst van attachments bij het downloaden. topdesk.py -> download_attachment()")
-		response = self.make_request("GET", f"incidents/id/{incident_id}/attachments/{attachment_id}/download", auto_parse=False)
+			raise FileNotFoundError(
+				"Attachment is niet gevonden tussen de lijst van attachments bij het downloaden. topdesk.py -> download_attachment()")
+		response = self.make_request("GET", f"incidents/id/{incident_id}/attachments/{attachment_id}/download",
+									 auto_parse=False)
 		return response.content, filename
 
 	def create_incident(
@@ -387,7 +404,7 @@ class TopdeskConnector:
 				"name": f"{priority}"
 			}
 		if costs:
-			body['costs'] =  f"{costs}"
+			body['costs'] = f"{costs}"
 		if category_name:
 			body["category"] = {
 				"name": f"{category_name}"
@@ -454,9 +471,10 @@ class TopdeskConnector:
 		}
 		return self.make_request('get', 'operationalActivities', params=params).get('result')
 
-	def operational_activity(self, activity_id: str) -> dict:
+	def operational_activity_with_status(self, activity_id: str) -> dict:
 		"""
-		Verkrijg informatie over 1 operational activity
+		Verkrijg informatie over 1 operational activity. Deze is specifiek gemaakt omdat er geen headers meegestuurd
+		moeten worden om de status te verkrijgen...
 
 		Args:
 			activity_id:
@@ -464,18 +482,112 @@ class TopdeskConnector:
 		Returns: dict
 
 		"""
-		return self.make_request('get', f'operationalActivities/{activity_id}/')
+		headers = self.headers.copy()
+		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
+		params = {"fields": "all"}
+		return self.make_request('get', f'operationalActivities/{activity_id}/', headers=headers, params=params)
+
+	def operational_activity(self, activity_id):
+		"""
+		Omdat de topdesk versie heel guur is, verschilt de output afhankelijk van welke header je gebruikt daaorm moet
+		deze informatie apart opgehaald worden. Zonder Accept header (belangrijk!)
+		Args:
+			activity_id:
+
+		Returns:
+
+		"""
+		return self.make_request('get', f'operationalActivities/{activity_id}')
+
+	def operational_activity_add_action(self, activity_id: str, text: str, flag: bool=False):
+		"""
+		Creeer een action bij een operationele activiteit.
+
+		Args:
+			activity_id: id van de operationele activiteit
+			text: Text van de actie.
+			flag: Markeer de action met een vlag.
+
+		Returns: str. Id van de actie die is aangemaakt.
+		"""
+		body = {
+			"memoText": text,
+			"flag": flag
+		}
+		return self.make_request('post', f'operationalActivities/{activity_id}/actions', json_data=body)
+
+	def operational_activity_update(self, activity_id: str, status = None, resolved=None, resolved_datetime=None, operator_id=None, operator_group_id=None):
+		"""
+		Update een bestaande operationele activiteit
+		Args:
+			activity_id: Id van de operationele activiteit
+			status: UUID van de nieuwe status van een status
+			resolved: boolean tickbox om de operationele activiteit als resolved te markeren
+			resolved_datetime: Optioneel datetime object voor resolved. Als datetime null is en resolved True, wordt datetime het moment van updaten.
+			operator_id: Optionele operator_id
+			operator_group_id: optionele operator_group_id
+
+		Returns: None
+
+		"""
+		if resolved_datetime is None:
+			resolved_datetime = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f%zZ")
+		else:
+			resolved_datetime = resolved_datetime.strftime("%Y-%m-%dT%H:%M:%S.%f%zZ")
+
+		body = {}
+		if status:
+			body['status'] = status
+		if resolved:
+			body['resolved'] = {
+				'value': True,
+				'dateTime': resolved_datetime
+			}
+		if operator_id:
+			body['operator'] = operator_id
+		if operator_group_id:
+			body['operatorGroup'] = operator_group_id
+
+		return self.make_request('POST', f'operationalActivities/{activity_id}', json_data=body)
+
+	def operational_activity_by_supplier(self, supplier_id: str, status_string: str, resolved: bool or None = None):
+		"""
+		Verkrijg operationele activiteiten op basis van supplier_id en status string.
+		De API van topdesk is best guur in dit opzicht. de call vereist de Accept: application/x.topdesk-om-activity-v1+json header
+		maar wanneer je deze set, krijg je alleen de id en number field terug. Dus moeten we deze nogmaals handmatig opvragen
+		om over alle informatie te beschikken.
+		Args:
+			supplier_id: id van de supplier gekoppeld aan de operationele activiteit.
+			status_string: string value van de status waarop gefilterd wordt
+			resolved: filter op resolved of niet.
+		"""
+		params = f"query=supplier.id=={supplier_id};type.name=={status_string}"
+		if resolved is not None:
+			params += f";resolved.value=={resolved}"
+		headers = self.headers.copy()
+		headers['Accept'] = "application/x.topdesk-om-activity-v1+json"
+		result = self.make_request('get', 'operationalActivities', params=params, headers=headers)
+		output = []
+		for activity in result['results']:
+			output.append(self.make_request('get', f'operationalActivities/{activity.get("id")}'))
+		return output
 
 	def operational_activity_upload_file(self, activity_id, file):
+		"""
+		UPload een file naar een operationele activiteit.
+		Args:
+			activity_id:
+			file:
+
+		Returns:
+
+		"""
 		headers = {
 			'Authorization': self.authkey,
 			'Accept': 'application/json',
 		}
-		params = {
-			'identifier': activity_id
-		}
 		files = {'file': file}
-		return self.make_request('POST', 'assetmgmt/uploads', headers=headers, params=params, files=files)
+		return self.make_request('POST', f'operationalActivities/{activity_id}/attachments/upload', headers=headers, files=files)
 
 	# **************************
 	# **** Asset management ****
@@ -519,6 +631,15 @@ class TopdeskConnector:
 		files = {'file': file}
 		return self.make_request('POST', 'assetmgmt/uploads', headers=headers, params=params, files=files)
 
+	def asset_uploaded_files(self, asset_id):
+		"""
+		Return een lijst met uploads toebehorende aan een asset.
+		"""
+		params = {
+			'assetId': asset_id
+		}
+		return self.make_request('GET', 'assetmgmt/uploads', params=params).get('dataSet')
+
 	def assets_as_tuples(self):
 		"""
 		Get all assets as a tuple of id and name
@@ -527,6 +648,21 @@ class TopdeskConnector:
 		assets = self.make_request('GET', 'assetmgmt/assets/')
 		return [(asset.get('id'), asset.get('text')) for asset in assets.get('dataSet')]
 
+	def asset_download_file(self, asset_id, attachment_id):
+		downloads = self.asset_uploaded_files(asset_id)
+		# Vind de juiste URL bij de attachment. Er is momenteel geen endpoint voor directe download.
+		for download in downloads:
+			if download['id'] == attachment_id:
+				url = download['url']
+				# De url moet de eerste twee delen van de URL verwijderen. dus explode op /
+				url = url.split('/')
+				# Verwijder de eerste twee delen.
+				url = url[3::]
+				# Join ze weer bij elkaar met slashes. Maar zonder laatste slash
+				url = "".join([el + "/" for el in url])[:-1:]
+				return self.make_request('GET', url, auto_parse=False)
+		return None
+
 	def assets_name_as_tuple(self):
 		"""
 		Get all assets as a tuple of name and name
